import traceback

from cpl_core.application import ApplicationABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console
from cpl_core.dependency_injection import ServiceProviderABC

from py_to_uxf_core.abc.python_parser_abc import PythonParserABC
from py_to_uxf_core.abc.umlet_creator_abc import UmletCreatorABC
from py_to_uxf_core.model.python_class import PythonClass


class Application(ApplicationABC):

    def __init__(self, config: ConfigurationABC, services: ServiceProviderABC):
        ApplicationABC.__init__(self, config, services)

        self._path = config.get_configuration('p')
        self._file = ''
        try:
            self._file = config.get_configuration('-pAdditionalArguments')[1]
        except Exception as e:
            Console.error('Expected file')
            exit()
        self._parser: PythonParserABC = services.get_service(PythonParserABC)
        self._umlet_creator: UmletCreatorABC = services.get_service(UmletCreatorABC)

    def configure(self):
        pass

    @staticmethod
    def _console_output(classes: list[PythonClass]):
        for cls in classes:
            Console.write_line(f'Class {cls.name}')
            if len(cls.attributes) > 0:
                Console.write_line('\tAttributes:')
            for atr in cls.attributes:
                Console.write_line(f'\t{atr.access_modifier.value}{atr.name}: {atr.type}')

            if len(cls.functions) > 0:
                Console.write_line('\tFunctions:')
            for func in cls.functions:
                args = ''
                Console.write_line(f'\t{func.access_modifier.value}{func.name}({args}): {func.return_type}')
            Console.write_line()

    def main(self):
        Console.banner('Py to UXF')
        if self._path is None:
            Console.error('Expected path')
            return

        if self._file is None:
            Console.error(f'Expected output file')
            return

        Console.write_line(f'Input path:', self._path)
        Console.write_line(f'Output path:', self._file)
        xml = ''
        try:
            classes: list[PythonClass] = self._parser.parse()
            Console.write_line(f'Found {len(classes)} classes')
            # self._console_output(classes)
            xml = self._umlet_creator.generate_xml(classes)
        except Exception as e:
            Console.error('Parsing failed', f'{e} -> {traceback.format_exc()}')
            exit()

        if not self._file.endswith('.uxf'):
            Console.error(f'Unexpected file {self._file}')
            exit()

        with open(self._file, 'w+') as file:
            file.write(xml)
            file.close()
